@@ -36,42 +36,24 @@
 #include <gdk-pixbuf/gdk-pixbuf-loader.h>
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
-#include <libgnomevfs/gnome-vfs-async-ops.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gio/gio.h>
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <pango/pangoft2.h>
 
 #define LOAD_BUFFER_SIZE 65536
 
-const ArtIRect eel_gdk_pixbuf_whole_pixbuf = { G_MININT, G_MININT, G_MAXINT, G_MAXINT };
+const EelIRect eel_gdk_pixbuf_whole_pixbuf = { G_MININT, G_MININT, G_MAXINT, G_MAXINT };
 
 struct EelPixbufLoadHandle {
-	GnomeVFSAsyncHandle *vfs_handle;
+	GCancellable *cancellable;
+	GInputStream *stream;
 	EelPixbufLoadCallback callback;
 	gpointer callback_data;
 	GdkPixbufLoader *loader;
 	char buffer[LOAD_BUFFER_SIZE];
 };
 
-static void file_opened_callback (GnomeVFSAsyncHandle *vfs_handle,
-				  GnomeVFSResult       result,
-				  gpointer             callback_data);
-static void file_read_callback   (GnomeVFSAsyncHandle *vfs_handle,
-				  GnomeVFSResult       result,
-				  gpointer             buffer,
-				  GnomeVFSFileSize     bytes_requested,
-				  GnomeVFSFileSize     bytes_read,
-				  gpointer             callback_data);
-static void file_closed_callback (GnomeVFSAsyncHandle *handle,
-				  GnomeVFSResult       result,
-				  gpointer             callback_data);
-static void load_done            (EelPixbufLoadHandle *handle,
-				  GnomeVFSResult       result,
-				  gboolean             get_pixbuf);
-
 /**
  * eel_gdk_pixbuf_list_ref
  * @pixbuf_list: A list of GdkPixbuf objects.
@@ -99,161 +81,128 @@ eel_gdk_pixbuf_list_free (GList *pixbuf_list)
 GdkPixbuf *
 eel_gdk_pixbuf_load (const char *uri)
 {
-	GnomeVFSResult result;
-	GnomeVFSHandle *handle;
-	char buffer[LOAD_BUFFER_SIZE];
-	GnomeVFSFileSize bytes_read;
-	GdkPixbufLoader *loader;
-	GdkPixbuf *pixbuf;	
+	GdkPixbuf *pixbuf;
+	GFile *file;
+	GFileInputStream *stream;
 
 	g_return_val_if_fail (uri != NULL, NULL);
 
-	result = gnome_vfs_open (&handle,
-				 uri,
-				 GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
+	file = g_file_new_for_uri (uri);
+
+	stream = g_file_read (file, NULL, NULL);
+	
+	g_object_unref (file);
+	
+	if (stream == NULL) {
 		return NULL;
 	}
 
+	pixbuf = eel_gdk_pixbuf_load_from_stream (G_INPUT_STREAM (stream));
+
+	g_object_unref (stream);
+		
+	return pixbuf;
+}
+
+GdkPixbuf *
+eel_gdk_pixbuf_load_from_stream (GInputStream  *stream)
+{
+	return eel_gdk_pixbuf_load_from_stream_at_size (stream, -1);
+}
+
+static void
+pixbuf_loader_size_prepared (GdkPixbufLoader *loader,
+			     int              width,
+			     int              height,
+			     gpointer         desired_size_ptr)
+{
+	int size, desired_size;
+	float scale;
+
+	size = MAX (width, height);
+	desired_size = GPOINTER_TO_INT (desired_size_ptr);
+
+	if (size != desired_size) {
+		scale = (float) desired_size / size;
+		gdk_pixbuf_loader_set_size (loader,
+					    floor (scale * width + 0.5),
+					    floor (scale * height + 0.5));
+	}
+}
+
+GdkPixbuf *
+eel_gdk_pixbuf_load_from_stream_at_size (GInputStream  *stream,
+					 int            size)
+{
+	char buffer[LOAD_BUFFER_SIZE];
+	gssize bytes_read;
+	GdkPixbufLoader *loader;
+	GdkPixbuf *pixbuf;
+	gboolean got_eos;
+	
+
+	g_return_val_if_fail (stream != NULL, NULL);
+
+	got_eos = FALSE;
 	loader = gdk_pixbuf_loader_new ();
+
+	if (size > 0) {
+		g_signal_connect (loader, "size-prepared",
+				  G_CALLBACK (pixbuf_loader_size_prepared),
+				  GINT_TO_POINTER (size));
+	}
+
 	while (1) {
-		result = gnome_vfs_read (handle,
-					 buffer,
-					 sizeof (buffer),
-					 &bytes_read);
-		if (result != GNOME_VFS_OK) {
+		bytes_read = g_input_stream_read (stream, buffer, sizeof (buffer),
+						  NULL, NULL);
+		
+		if (bytes_read < 0) {
 			break;
 		}
 		if (bytes_read == 0) {
+			got_eos = TRUE;
 			break;
 		}
 		if (!gdk_pixbuf_loader_write (loader,
 					      buffer,
 					      bytes_read,
 					      NULL)) {
-			result = GNOME_VFS_ERROR_WRONG_FORMAT;
 			break;
 		}
 	}
 
-	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-		gdk_pixbuf_loader_close (loader, NULL);
-		g_object_unref (loader);
-		gnome_vfs_close (handle);
-		return NULL;
-	}
-
-	gnome_vfs_close (handle);
+	g_input_stream_close (stream, NULL, NULL);
 	gdk_pixbuf_loader_close (loader, NULL);
 
-	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-	if (pixbuf != NULL) {
-		g_object_ref (pixbuf);
+	pixbuf = NULL;
+	if (got_eos) {
+		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+		if (pixbuf != NULL) {
+			g_object_ref (pixbuf);
+		}
 	}
+
 	g_object_unref (loader);
 
 	return pixbuf;
 }
 
-EelPixbufLoadHandle *
-eel_gdk_pixbuf_load_async (const char *uri,
-			   int priority,
-			   EelPixbufLoadCallback callback,
-			   gpointer callback_data)
-{
-	EelPixbufLoadHandle *handle;
-
-	handle = g_new0 (EelPixbufLoadHandle, 1);
-	handle->callback = callback;
-	handle->callback_data = callback_data;
-
-	gnome_vfs_async_open (&handle->vfs_handle,
-			      uri,
-			      GNOME_VFS_OPEN_READ,
-			      priority,
-			      file_opened_callback,
-			      handle);
-
-	return handle;
-}
-
-static void
-file_opened_callback (GnomeVFSAsyncHandle *vfs_handle,
-		      GnomeVFSResult result,
-		      gpointer callback_data)
-{
-	EelPixbufLoadHandle *handle;
-
-	handle = callback_data;
-	g_assert (handle->vfs_handle == vfs_handle);
-
-	if (result != GNOME_VFS_OK) {
-		handle->vfs_handle = NULL;
-		load_done (handle, result, FALSE);
-		return;
-	}
-
-	handle->loader = gdk_pixbuf_loader_new ();
-
-	gnome_vfs_async_read (handle->vfs_handle,
-			      handle->buffer,
-			      sizeof (handle->buffer),
-			      file_read_callback,
-			      handle);
-}
-
-static void
-file_read_callback (GnomeVFSAsyncHandle *vfs_handle,
-		    GnomeVFSResult result,
-		    gpointer buffer,
-		    GnomeVFSFileSize bytes_requested,
-		    GnomeVFSFileSize bytes_read,
-		    gpointer callback_data)
-{
-	EelPixbufLoadHandle *handle;
-
-	handle = callback_data;
-	g_assert (handle->vfs_handle == vfs_handle);
-	g_assert (handle->buffer == buffer);
-
-	if (result == GNOME_VFS_OK && bytes_read != 0) {
-		if (!gdk_pixbuf_loader_write (handle->loader,
-					      buffer,
-					      bytes_read,
-					      NULL)) {
-			result = GNOME_VFS_ERROR_WRONG_FORMAT;
-		} else {
-			gnome_vfs_async_read (handle->vfs_handle,
-					      handle->buffer,
-					      sizeof (handle->buffer),
-					      file_read_callback,
-					      handle);
-			return;
-		}
-	}
-
-	load_done (handle, result, result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_EOF);
-}
-
-static void
-file_closed_callback (GnomeVFSAsyncHandle *handle,
-		      GnomeVFSResult result,
-		      gpointer callback_data)
-{
-	g_assert (callback_data == NULL);
-}
-
 static void
 free_pixbuf_load_handle (EelPixbufLoadHandle *handle)
 {
+	g_object_unref (handle->cancellable);
 	if (handle->loader != NULL) {
 		g_object_unref (handle->loader);
 	}
+	if (handle->stream) {
+		g_input_stream_close_async (handle->stream, 0, NULL, NULL, NULL);
+		g_object_unref (handle->stream);
+	}
 	g_free (handle);
 }
 
 static void
-load_done (EelPixbufLoadHandle *handle, GnomeVFSResult result, gboolean get_pixbuf)
+load_done (EelPixbufLoadHandle *handle, GError *error, gboolean get_pixbuf)
 {
 	GdkPixbuf *pixbuf;
 
@@ -263,28 +212,122 @@ load_done (EelPixbufLoadHandle *handle, GnomeVFSResult result, gboolean get_pixb
 
 	pixbuf = get_pixbuf ? gdk_pixbuf_loader_get_pixbuf (handle->loader) : NULL;
 	
-	if (handle->vfs_handle != NULL) {
-		gnome_vfs_async_close (handle->vfs_handle, file_closed_callback, NULL);
-	}
-
-	handle->callback (result, pixbuf, handle->callback_data);
+	handle->callback (error, pixbuf, handle->callback_data);
 
 	free_pixbuf_load_handle (handle);
 }
 
+static void
+file_read_callback (GObject *source_object,
+		    GAsyncResult *res,
+		    gpointer user_data)
+{
+	EelPixbufLoadHandle *handle;
+	gssize bytes_read;
+	GError *error;
+
+	handle = user_data;
+
+	if (g_cancellable_is_cancelled (handle->cancellable)) {
+		free_pixbuf_load_handle (handle);
+		return;
+	}
+	
+	error = NULL;
+	bytes_read = g_input_stream_read_finish  (G_INPUT_STREAM (source_object),
+						  res, &error);
+	
+	if (bytes_read > 0) {
+		if (!gdk_pixbuf_loader_write (handle->loader,
+					      handle->buffer,
+					      bytes_read,
+					      &error)) {
+			bytes_read = -1;
+		} else {
+			g_input_stream_read_async (handle->stream,
+						   handle->buffer,
+						   sizeof (handle->buffer),
+						   0,
+						   handle->cancellable,
+						   file_read_callback, handle);
+			return;
+		}
+	}
+
+	load_done (handle, error, bytes_read == 0);
+	
+	if (error != NULL) {
+		g_error_free (error);
+	}
+}
+
+static void
+file_opened_callback (GObject *source_object,
+		      GAsyncResult *res,
+		      gpointer user_data)
+{
+	EelPixbufLoadHandle *handle;
+	GFileInputStream *stream;
+	GError *error;
+
+	handle = user_data;
+
+	if (g_cancellable_is_cancelled (handle->cancellable)) {
+		free_pixbuf_load_handle (handle);
+		return;
+	}
+
+	error = NULL;
+	stream = g_file_read_finish (G_FILE (source_object), res, &error);
+
+	if (stream == NULL) {
+		load_done (handle, error, FALSE);
+		g_error_free (error);
+		return;
+	}
+
+	handle->stream = G_INPUT_STREAM (stream);
+	handle->loader = gdk_pixbuf_loader_new ();
+
+	
+	g_input_stream_read_async (handle->stream,
+				   handle->buffer,
+				   sizeof (handle->buffer),
+				   0,
+				   handle->cancellable,
+				   file_read_callback, handle);
+}
+
+EelPixbufLoadHandle *
+eel_gdk_pixbuf_load_async (const char *uri,
+			   int priority,
+			   EelPixbufLoadCallback callback,
+			   gpointer callback_data)
+{
+	EelPixbufLoadHandle *handle;
+	GFile *file;
+
+	handle = g_new0 (EelPixbufLoadHandle, 1);
+	handle->cancellable = g_cancellable_new ();
+	handle->callback = callback;
+	handle->callback_data = callback_data;
+
+	file = g_file_new_for_uri (uri);
+	
+	g_file_read_async (file, priority, handle->cancellable,
+			   file_opened_callback, handle);
+
+	return handle;
+}
+
 void
 eel_cancel_gdk_pixbuf_load (EelPixbufLoadHandle *handle)
 {
 	if (handle == NULL) {
 		return;
 	}
-	if (handle->loader != NULL) {
-		gdk_pixbuf_loader_close (handle->loader, NULL);
-	}
-	if (handle->vfs_handle != NULL) {
-		gnome_vfs_async_cancel (handle->vfs_handle);
-	}
-	free_pixbuf_load_handle (handle);
+	
+	g_cancellable_cancel (handle->cancellable);
 }
 
 /* return the average value of each component */
@@ -404,6 +447,37 @@ eel_gdk_pixbuf_scale_down_to_fit (GdkPixbuf *pixbuf, int max_width, int max_heig
 	}
 }
 
+double
+eel_gdk_scale_to_min_factor (int width, int height,
+			     int min_width, int min_height,
+			     int *scaled_width, int *scaled_height)
+{
+	double scale_factor;
+
+	scale_factor = MAX (min_width / (double) width, min_height / (double) height);
+
+	*scaled_width  = floor (width * scale_factor + .5);
+	*scaled_height = floor (height * scale_factor + .5);
+
+	return scale_factor;
+}
+
+/* Returns a scaled copy of pixbuf, preserving aspect ratio. The copy will
+ * be scaled as small as possible without going under the specified width and height.
+ */
+GdkPixbuf *
+eel_gdk_pixbuf_scale_to_min (GdkPixbuf *pixbuf, int min_width, int min_height)
+{
+	int scaled_width;
+	int scaled_height;
+
+	eel_gdk_scale_to_min_factor (gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf),
+				     min_width, min_height,
+				     &scaled_width, &scaled_height);
+
+	return gdk_pixbuf_scale_simple (pixbuf, scaled_width, scaled_height, GDK_INTERP_BILINEAR);
+}
+
 /**
  * eel_gdk_pixbuf_is_valid:
  * @pixbuf: A GdkPixbuf
@@ -457,10 +531,10 @@ eel_gdk_pixbuf_get_dimensions (const GdkPixbuf *pixbuf)
  */
 void
 eel_gdk_pixbuf_fill_rectangle_with_color (GdkPixbuf *pixbuf,
-					  ArtIRect area,
+					  EelIRect area,
 					  guint32 color)
 {
-	ArtIRect target;
+	EelIRect target;
 	guchar red;
 	guchar green;
 	guchar blue;
@@ -476,7 +550,7 @@ eel_gdk_pixbuf_fill_rectangle_with_color (GdkPixbuf *pixbuf,
 	g_return_if_fail (eel_gdk_pixbuf_is_valid (pixbuf));
 	
 	target = eel_gdk_pixbuf_intersect (pixbuf, 0, 0, area);
-	if (art_irect_empty (&target)) {
+	if (eel_irect_is_empty (&target)) {
 		return;
 	}
 
@@ -539,14 +613,14 @@ eel_gdk_pixbuf_draw_to_drawable (const GdkPixbuf *pixbuf,
 				 GdkGC *gc,
 				 int source_x,
 				 int source_y,
-				 ArtIRect destination_area,
+				 EelIRect destination_area,
 				 GdkRgbDither dither,
 				 GdkPixbufAlphaMode alpha_compositing_mode,
 				 int alpha_threshold)
 {
 	EelDimensions dimensions;
-	ArtIRect target;
-	ArtIRect source;
+	EelIRect target;
+	EelIRect source;
 	int target_width;
 	int target_height;
 	int source_width;
@@ -555,7 +629,7 @@ eel_gdk_pixbuf_draw_to_drawable (const GdkPixbuf *pixbuf,
 	g_return_if_fail (eel_gdk_pixbuf_is_valid (pixbuf));
 	g_return_if_fail (drawable != NULL);
 	g_return_if_fail (gc != NULL);
-	g_return_if_fail (!art_irect_empty (&destination_area));
+	g_return_if_fail (!eel_irect_is_empty (&destination_area));
  	g_return_if_fail (alpha_threshold > EEL_OPACITY_FULLY_TRANSPARENT);
  	g_return_if_fail (alpha_threshold <= EEL_OPACITY_FULLY_OPAQUE);
  	g_return_if_fail (alpha_compositing_mode >= GDK_PIXBUF_ALPHA_BILEVEL);
@@ -573,12 +647,12 @@ eel_gdk_pixbuf_draw_to_drawable (const GdkPixbuf *pixbuf,
 					   destination_area.x0,
 					   destination_area.y0,
 					   destination_area);
-	if (art_irect_empty (&target)) {
+	if (eel_irect_is_empty (&target)) {
 		return;
 	}
 
 	/* Assign the source area */
-	source = eel_art_irect_assign (source_x,
+	source = eel_irect_assign (source_x,
 				       source_y,
 				       dimensions.width - source_x,
 				       dimensions.height - source_y);
@@ -631,11 +705,11 @@ eel_gdk_pixbuf_draw_to_pixbuf (const GdkPixbuf *pixbuf,
 			       GdkPixbuf *destination_pixbuf,
 			       int source_x,
 			       int source_y,
-			       ArtIRect destination_area)
+			       EelIRect destination_area)
 {
 	EelDimensions dimensions;
-	ArtIRect target;
-	ArtIRect source;
+	EelIRect target;
+	EelIRect source;
 	int target_width;
 	int target_height;
 	int source_width;
@@ -643,7 +717,7 @@ eel_gdk_pixbuf_draw_to_pixbuf (const GdkPixbuf *pixbuf,
 	
 	g_return_if_fail (eel_gdk_pixbuf_is_valid (pixbuf));
 	g_return_if_fail (eel_gdk_pixbuf_is_valid (destination_pixbuf));
-	g_return_if_fail (!art_irect_empty (&destination_area));
+	g_return_if_fail (!eel_irect_is_empty (&destination_area));
 
 	dimensions = eel_gdk_pixbuf_get_dimensions (pixbuf);
 
@@ -654,12 +728,12 @@ eel_gdk_pixbuf_draw_to_pixbuf (const GdkPixbuf *pixbuf,
 
 	/* Clip the destination area to the pixbuf dimensions; bail if no work */
 	target = eel_gdk_pixbuf_intersect (destination_pixbuf, 0, 0, destination_area);
-	if (art_irect_empty (&target)) {
+	if (eel_irect_is_empty (&target)) {
  		return;
  	}
 
 	/* Assign the source area */
-	source = eel_art_irect_assign (source_x,
+	source = eel_irect_assign (source_x,
 				       source_y,
 				       dimensions.width - source_x,
 				       dimensions.height - source_y);
@@ -715,13 +789,13 @@ eel_gdk_pixbuf_draw_to_pixbuf_alpha (const GdkPixbuf *pixbuf,
 				     GdkPixbuf *destination_pixbuf,
 				     int source_x,
 				     int source_y,
-				     ArtIRect destination_area,
+				     EelIRect destination_area,
 				     int opacity,
 				     GdkInterpType interpolation_mode)
 {
 	EelDimensions dimensions;
-	ArtIRect target;
-	ArtIRect source;
+	EelIRect target;
+	EelIRect source;
 	int target_width;
 	int target_height;
 	int source_width;
@@ -729,7 +803,7 @@ eel_gdk_pixbuf_draw_to_pixbuf_alpha (const GdkPixbuf *pixbuf,
 
 	g_return_if_fail (eel_gdk_pixbuf_is_valid (pixbuf));
 	g_return_if_fail (eel_gdk_pixbuf_is_valid (destination_pixbuf));
-	g_return_if_fail (!art_irect_empty (&destination_area));
+	g_return_if_fail (!eel_irect_is_empty (&destination_area));
 	g_return_if_fail (opacity >= EEL_OPACITY_FULLY_TRANSPARENT);
 	g_return_if_fail (opacity <= EEL_OPACITY_FULLY_OPAQUE);
 	g_return_if_fail (interpolation_mode >= GDK_INTERP_NEAREST);
@@ -744,12 +818,12 @@ eel_gdk_pixbuf_draw_to_pixbuf_alpha (const GdkPixbuf *pixbuf,
 
 	/* Clip the destination area to the pixbuf dimensions; bail if no work */
 	target = eel_gdk_pixbuf_intersect (destination_pixbuf, 0, 0, destination_area);
-	if (art_irect_empty (&target)) {
+	if (eel_irect_is_empty (&target)) {
  		return;
  	}
 
 	/* Assign the source area */
-	source = eel_art_irect_assign (source_x,
+	source = eel_irect_assign (source_x,
 				       source_y,
 				       dimensions.width - source_x,
 				       dimensions.height - source_y);
@@ -768,7 +842,7 @@ eel_gdk_pixbuf_draw_to_pixbuf_alpha (const GdkPixbuf *pixbuf,
 	 * with only the source area.  This is needed to work around a limitation
 	 * in gdk_pixbuf_composite() that requires the source area to be (0,0). */
 	if (source.x0 != 0 || source.y0 != 0) {
-		ArtIRect area;
+		EelIRect area;
 		int width;
 		int height;
 
@@ -805,8 +879,8 @@ static void
 pixbuf_destroy_callback (guchar  *pixels,
 			 gpointer callback_data)
 {
-	g_return_if_fail (pixels != NULL);
-	g_return_if_fail (callback_data != NULL);
+	g_assert (pixels != NULL);
+	g_assert (callback_data != NULL);
 
 	g_object_unref (callback_data);
 }
@@ -828,18 +902,18 @@ pixbuf_destroy_callback (guchar  *pixels,
  */
 GdkPixbuf *
 eel_gdk_pixbuf_new_from_pixbuf_sub_area (GdkPixbuf *pixbuf,
-					 ArtIRect area)
+					 EelIRect area)
 {
 	GdkPixbuf *sub_pixbuf;
-	ArtIRect target;
+	EelIRect target;
 	guchar *pixels;
 	
 	g_return_val_if_fail (eel_gdk_pixbuf_is_valid (pixbuf), NULL);
-	g_return_val_if_fail (!art_irect_empty (&area), NULL);
+	g_return_val_if_fail (!eel_irect_is_empty (&area), NULL);
 
 	/* Clip the pixbuf by the given area; bail if no work */
 	target = eel_gdk_pixbuf_intersect (pixbuf, 0, 0, area);
-	if (art_irect_empty (&target)) {
+	if (eel_irect_is_empty (&target)) {
  		return NULL;
  	}
 
@@ -858,8 +932,8 @@ eel_gdk_pixbuf_new_from_pixbuf_sub_area (GdkPixbuf *pixbuf,
 					       GDK_COLORSPACE_RGB,
 					       gdk_pixbuf_get_has_alpha (pixbuf),
 					       8,
-					       eel_art_irect_get_width (target),
-					       eel_art_irect_get_height (target),
+					       eel_irect_get_width (target),
+					       eel_irect_get_height (target),
 					       gdk_pixbuf_get_rowstride (pixbuf),
 					       pixbuf_destroy_callback,
 					       pixbuf);
@@ -890,14 +964,14 @@ GdkPixbuf *
 eel_gdk_pixbuf_new_from_existing_buffer (guchar *buffer,
 					 int buffer_rowstride,
 					 gboolean buffer_has_alpha,
-					 ArtIRect area)
+					 EelIRect area)
 {
 	GdkPixbuf *pixbuf;
 	guchar *pixels;
 	
 	g_return_val_if_fail (buffer != NULL, NULL);
 	g_return_val_if_fail (buffer_rowstride > 0, NULL);
-	g_return_val_if_fail (!art_irect_empty (&area), NULL);
+	g_return_val_if_fail (!eel_irect_is_empty (&area), NULL);
 	
 	/* Compute the offset into the buffer */
 	pixels = 
@@ -909,8 +983,8 @@ eel_gdk_pixbuf_new_from_existing_buffer (guchar *buffer,
 					   GDK_COLORSPACE_RGB,
 					   buffer_has_alpha,
 					   8,
-					   eel_art_irect_get_width (area),
-					   eel_art_irect_get_height (area),
+					   eel_irect_get_width (area),
+					   eel_irect_get_height (area),
 					   buffer_rowstride,
 					   NULL,
 					   NULL);
@@ -918,457 +992,45 @@ eel_gdk_pixbuf_new_from_existing_buffer (guchar *buffer,
 	return pixbuf;
 }
 
-/* The tile algorithm is identical whether the destination is 
- * a pixbuf or a drawable.  So, we use a simple callback
- * mechanism to share it regardless of the destination.
- */
-typedef struct {
-	GdkPixbuf *destination_pixbuf;
-	int opacity;
-	GdkInterpType interpolation_mode;
-} PixbufTileData;
-
-typedef struct {
-	GdkDrawable *drawable;
-	GdkGC *gc;
-	GdkRgbDither dither;
-	GdkPixbufAlphaMode alpha_compositing_mode;
-	int alpha_threshold;
-} DrawableTileData;
-
-typedef void (* DrawPixbufTileCallback) (const GdkPixbuf *pixbuf,
-					 int x,
-					 int y,
-					 ArtIRect destination_area,
-					 gpointer callback_data);
-
-/* The shared tiliing implementation */
-static void
-pixbuf_draw_tiled (const GdkPixbuf *pixbuf,
-		   EelDimensions destination_dimensions,
-		   ArtIRect destination_area,
-		   int tile_width,
-		   int tile_height,
-		   int tile_origin_x,
-		   int tile_origin_y,
-		   DrawPixbufTileCallback callback,
-		   gpointer callback_data)
-{
-	ArtIRect target;
-	int x;
-	int y;
-	EelArtIPoint min_point;
-	EelArtIPoint max_point;
-	int num_left;
-	int num_above;
-	ArtIRect clipped_destination_area;
-
-	g_return_if_fail (pixbuf != NULL);
-	g_return_if_fail (destination_dimensions.width > 0);
-	g_return_if_fail (destination_dimensions.height > 0);
-	g_return_if_fail (tile_width > 0);
-	g_return_if_fail (tile_height > 0);
-	g_return_if_fail (tile_width <= gdk_pixbuf_get_width (pixbuf));
-	g_return_if_fail (tile_height <= gdk_pixbuf_get_height (pixbuf));
-	g_return_if_fail (callback != NULL);
-	g_return_if_fail (!art_irect_empty (&destination_area));
-
-	/* FIXME: This is confusing.  Instead of passing in the destination_dimensions
-	 *        I should just pass in the destination pixbuf, so that we can use
-	 *        eel_gdk_pixbuf_intersect directly on that.
-	 */
-	clipped_destination_area = eel_art_irect_assign_dimensions (0, 0, destination_dimensions);
-	art_irect_intersect (&target, &destination_area, &clipped_destination_area);
-	if (art_irect_empty (&target)) {
-		return;
-	}
-
-	/* The number of tiles left and above the target area */
-	num_left = (target.x0 - tile_origin_x) / tile_width;
-	num_above = (target.y0 - tile_origin_y) / tile_height;
-	
-	min_point.x = tile_origin_x - tile_width + (num_left * tile_width);
-	min_point.y = tile_origin_y - tile_height + (num_above * tile_height);
-	
-	max_point.x = (target.x1 + 2 * tile_width);
-	max_point.y = (target.y1 + 2 * tile_height);
-	
-	for (y = min_point.y; y <= max_point.y; y += tile_height) {
-		for (x = min_point.x; x <= max_point.x; x += tile_width) {
-			ArtIRect current;
-			ArtIRect area;
-
-			current = eel_art_irect_assign (x, y, tile_width, tile_height);
-
-			/* FIXME: A potential speed improvement here would be to clip only the
-			 * first and last rectangles, not the ones in between.  
-			 */
-			art_irect_intersect (&area, &target, &current);
-
-			if (!art_irect_empty (&area)) {
-				g_assert (area.x0 >= x);
-				g_assert (area.y0 >= y);
-
-				(* callback) (pixbuf,
-					      area.x0 - x,
-					      area.y0 - y,
-					      area,
-					      callback_data);
-			}
-		}
-	}
-}
-
-static void
-draw_tile_to_pixbuf_callback (const GdkPixbuf *pixbuf,
-			      int x,
-			      int y,
-			      ArtIRect area,
-			      gpointer callback_data)
-{
-	PixbufTileData *pixbuf_tile_data;
-
-	g_return_if_fail (pixbuf != NULL);
-	g_return_if_fail (callback_data != NULL);
-	g_return_if_fail (!art_irect_empty (&area));
-
-	pixbuf_tile_data = (PixbufTileData *) callback_data;
-
-	if (pixbuf_tile_data->opacity == EEL_OPACITY_FULLY_TRANSPARENT) {
-		eel_gdk_pixbuf_draw_to_pixbuf (pixbuf,
-					       pixbuf_tile_data->destination_pixbuf,
-					       x,
-					       y,
-					       area);
-	} else {
-		eel_gdk_pixbuf_draw_to_pixbuf_alpha (pixbuf,
-						     pixbuf_tile_data->destination_pixbuf,
-						     x,
-						     y,
-						     area,
-						     pixbuf_tile_data->opacity,
-						     pixbuf_tile_data->interpolation_mode);
-	}
-}
-
-static void
-draw_tile_to_drawable_callback (const GdkPixbuf *pixbuf,
-				int x,
-				int y,
-				ArtIRect area,
-				gpointer callback_data)
-{
-	DrawableTileData *drawable_tile_data;
-	
-	g_return_if_fail (pixbuf != NULL);
-	g_return_if_fail (callback_data != NULL);
-	g_return_if_fail (!art_irect_empty (&area));
-
-	drawable_tile_data = (DrawableTileData *) callback_data;
-
-	eel_gdk_pixbuf_draw_to_drawable (pixbuf,
-					 drawable_tile_data->drawable,
-					 drawable_tile_data->gc,
-					 x,
-					 y,
-					 area,
-					 drawable_tile_data->dither,
-					 drawable_tile_data->alpha_compositing_mode,
-					 drawable_tile_data->alpha_threshold);
-}
-
-/**
- * eel_gdk_pixbuf_draw_to_pixbuf_tiled:
- * @pixbuf: Source tile pixbuf.
- * @destination_pixbuf: Destination pixbuf.
- * @destination_area: Area of the destination pixbuf to tile.
- * @tile_width: Width of the tile.  This can be less than width of the
- *              tile pixbuf, but not greater.
- * @tile_height: Height of the tile.  This can be less than width of the
- *               tile pixbuf, but not greater.
- * @tile_origin_x: The x coordinate of the tile origin.  Can be negative.
- * @tile_origin_y: The y coordinate of the tile origin.  Can be negative.
- * @opacity: The opacity of the drawn tiles where 0 <= opacity <= 255.
- * @interpolation_mode: The interpolation mode.  See <gdk-pixbuf.h>
- *
- * Fill an area of a GdkPixbuf with a tile.
- */
-void
-eel_gdk_pixbuf_draw_to_pixbuf_tiled (const GdkPixbuf *pixbuf,
-				     GdkPixbuf *destination_pixbuf,
-				     ArtIRect destination_area,
-				     int tile_width,
-				     int tile_height,
-				     int tile_origin_x,
-				     int tile_origin_y,
-				     int opacity,
-				     GdkInterpType interpolation_mode)
-{
-	PixbufTileData pixbuf_tile_data;
-	EelDimensions destination_dimensions;
-
-	g_return_if_fail (eel_gdk_pixbuf_is_valid (destination_pixbuf));
-	g_return_if_fail (eel_gdk_pixbuf_is_valid (pixbuf));
-	g_return_if_fail (tile_width > 0);
-	g_return_if_fail (tile_height > 0);
-	g_return_if_fail (tile_width <= gdk_pixbuf_get_width (pixbuf));
-	g_return_if_fail (tile_height <= gdk_pixbuf_get_height (pixbuf));
-	g_return_if_fail (opacity >= EEL_OPACITY_FULLY_TRANSPARENT);
-	g_return_if_fail (opacity <= EEL_OPACITY_FULLY_OPAQUE);
-	g_return_if_fail (interpolation_mode >= GDK_INTERP_NEAREST);
-	g_return_if_fail (interpolation_mode <= GDK_INTERP_HYPER);
-
-	destination_dimensions = eel_gdk_pixbuf_get_dimensions (destination_pixbuf);
-
-	pixbuf_tile_data.destination_pixbuf = destination_pixbuf;
-	pixbuf_tile_data.opacity = opacity;
-	pixbuf_tile_data.interpolation_mode = interpolation_mode;
-
-	pixbuf_draw_tiled (pixbuf,
-			   destination_dimensions,
-			   destination_area,
-			   tile_width,
-			   tile_height,
-			   tile_origin_x,
-			   tile_origin_y,
-			   draw_tile_to_pixbuf_callback,
-			   &pixbuf_tile_data);
-}
-
-/**
- * eel_gdk_pixbuf_draw_to_drawable_tiled:
- * @pixbuf: Source tile pixbuf.
- * @gc: GC for copy area operation.
- * @drawable: Destination drawable.
- * @destination_area: Area of the destination pixbuf to tile.
- * @tile_width: Width of the tile.  This can be less than width of the
- *              tile pixbuf, but not greater.
- * @tile_height: Height of the tile.  This can be less than width of the
- *               tile pixbuf, but not greater.
- * @tile_origin_x: The x coordinate of the tile origin.  Can be negative.
- * @tile_origin_y: The y coordinate of the tile origin.  Can be negative.
- * @dither: Dither type to use (see <gdkrgb.h>)
- * @dither: Dither type to use (see <gdkrgb.h>)
- * @alpha_compositing_mode: The alpha compositing mode.  See <gdk-pixbuf.h>
- *
- * Fill an area of a GdkDrawable with a tile.
- */
-void
-eel_gdk_pixbuf_draw_to_drawable_tiled (const GdkPixbuf *pixbuf,
-				       GdkDrawable *drawable,
-				       GdkGC *gc,
-				       ArtIRect destination_area,
-				       int tile_width,
-				       int tile_height,
-				       int tile_origin_x,
-				       int tile_origin_y,
-				       GdkRgbDither dither,
-				       GdkPixbufAlphaMode alpha_compositing_mode,
-				       int alpha_threshold)
-{
-	DrawableTileData drawable_tile_data;
-	EelDimensions destination_dimensions;
-
-	g_return_if_fail (eel_gdk_pixbuf_is_valid (pixbuf));
-	g_return_if_fail (drawable != NULL);
-	g_return_if_fail (tile_width > 0);
-	g_return_if_fail (tile_height > 0);
-	g_return_if_fail (tile_width <= gdk_pixbuf_get_width (pixbuf));
-	g_return_if_fail (tile_height <= gdk_pixbuf_get_height (pixbuf));
- 	g_return_if_fail (alpha_threshold > EEL_OPACITY_FULLY_TRANSPARENT);
- 	g_return_if_fail (alpha_threshold <= EEL_OPACITY_FULLY_OPAQUE);
- 	g_return_if_fail (alpha_compositing_mode >= GDK_PIXBUF_ALPHA_BILEVEL);
- 	g_return_if_fail (alpha_compositing_mode <= GDK_PIXBUF_ALPHA_FULL);
-
-	destination_dimensions = eel_gdk_window_get_dimensions (drawable);
-	
-	drawable_tile_data.drawable = drawable;
-	drawable_tile_data.gc = gc;
-	drawable_tile_data.dither = dither;
-	drawable_tile_data.alpha_compositing_mode = alpha_compositing_mode;
-	drawable_tile_data.alpha_threshold = alpha_threshold;
-	
-	pixbuf_draw_tiled (pixbuf,
-			   destination_dimensions,
-			   destination_area,
-			   tile_width,
-			   tile_height,
-			   tile_origin_x,
-			   tile_origin_y,
-			   draw_tile_to_drawable_callback,
-			   &drawable_tile_data);
-}
-
 /**
  * eel_gdk_pixbuf_intersect:
  * @pixbuf: A GdkPixbuf.
  * @pixbuf_x: X coordinate of pixbuf.
  * @pixbuf_y: Y coordinate of pixbuf.
- * @rectangle: An ArtIRect.
+ * @rectangle: An EelIRect.
  *
  * Return value: The intersection of the pixbuf and the given rectangle.
  *
  */
-ArtIRect
+EelIRect
 eel_gdk_pixbuf_intersect (const GdkPixbuf *pixbuf,
 			  int pixbuf_x,
 			  int pixbuf_y,
-			  ArtIRect rectangle)
+			  EelIRect rectangle)
 {
-	ArtIRect intersection;
-	ArtIRect bounds;
+	EelIRect intersection;
+	EelIRect bounds;
 	EelDimensions dimensions;
 
-	g_return_val_if_fail (eel_gdk_pixbuf_is_valid (pixbuf), eel_art_irect_empty);
+	g_return_val_if_fail (eel_gdk_pixbuf_is_valid (pixbuf), eel_irect_empty);
 
 	dimensions = eel_gdk_pixbuf_get_dimensions (pixbuf);
-	bounds = eel_art_irect_assign_dimensions (pixbuf_x, pixbuf_y, dimensions);
+	bounds = eel_irect_assign_dimensions (pixbuf_x, pixbuf_y, dimensions);
 
-	art_irect_intersect (&intersection, &rectangle, &bounds);
+	eel_irect_intersect (&intersection, &rectangle, &bounds);
 
 	/* In theory, this is not needed because a rectangle is empty
 	 * regardless of how MUCH negative the dimensions are.  
 	 * However, to make debugging and self checks simpler, we
 	 * consistenly return a standard empty rectangle.
 	 */
-	if (art_irect_empty (&intersection)) {
-		return eel_art_irect_empty;
+	if (eel_irect_is_empty (&intersection)) {
+		return eel_irect_empty;
 	}
 
 	return intersection;
 }
 
-/**
- * eel_gdk_pixbuf_draw_layout_clipped:
- * @pixbuf: the pixbuf to render to
- * @region: text crop region
- * @color: the color to render the text
- * @layout: the text
- * 
- * This method renders the text in @layout into the
- * @pixbuf with @color cropping it to @region.
- **/
-void
-eel_gdk_pixbuf_draw_layout_clipped (GdkPixbuf   *pixbuf,
-				    ArtIRect     region,
-				    guint32      color,
-				    PangoLayout *layout)
-{
-	PangoRectangle ink_rect;
-	FT_Bitmap ink_bitmap;
-	GdkPixbuf *ink_pixbuf;
-	int bx, by;
-	int pixbuf_width, pixbuf_height;
-	const guint8 *ps;
-	guint8 *pd;
-	guint8 r, g, b;
-
-	if (region.x1 <= region.x0 || region.y1 <= region.y0) {
-		return;
-	}
-	
-	pango_layout_get_pixel_extents (layout, &ink_rect, NULL);
-
-	ink_rect.x += region.x0;
-	ink_rect.y += region.y0;
-
-	/* Do an intersect operation the long way.
-	 * Sad to write this all out like this.
-	 */
-
-	if (ink_rect.x < 0) {
-		ink_rect.width += ink_rect.x;
-		ink_rect.x = 0;
-	}
-
-	if (ink_rect.y < 0) {
-		ink_rect.height += ink_rect.y;
-		ink_rect.y = 0;
-	}
-
-	pixbuf_width = gdk_pixbuf_get_width (pixbuf);
-	if (ink_rect.x + ink_rect.width > pixbuf_width) {
-		ink_rect.width = pixbuf_width - ink_rect.x;
-		if (ink_rect.width <= 0) {
-			return;
-		}
-	}
-
-	pixbuf_height = gdk_pixbuf_get_height (pixbuf);
-	if (ink_rect.y + ink_rect.height > pixbuf_height) {
-		ink_rect.height = pixbuf_height - ink_rect.y;
-		if (ink_rect.height <= 0) {
-			return;
-		}
-	}
-
-	ink_bitmap.rows = ink_rect.height;
-	ink_bitmap.width = ink_rect.width;
-	ink_bitmap.pitch = (ink_bitmap.width + 3) & ~3;
-	ink_bitmap.buffer = g_malloc0 (ink_bitmap.rows * ink_bitmap.pitch);
-	ink_bitmap.num_grays = 0x100;
-	ink_bitmap.pixel_mode = ft_pixel_mode_grays;
-
-	pango_ft2_render_layout (&ink_bitmap, layout,
-				 region.x0 - ink_rect.x,
-				 region.y0 - ink_rect.y);
-
-	ink_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				     ink_bitmap.width, ink_bitmap.rows);
-
-	r = EEL_RGBA_COLOR_GET_R (color);
-	g = EEL_RGBA_COLOR_GET_G (color);
-	b = EEL_RGBA_COLOR_GET_B (color);
-	
-	for (by = 0; by < ink_bitmap.rows; by++) {
-	        pd = gdk_pixbuf_get_pixels (ink_pixbuf)
-			+ by * gdk_pixbuf_get_rowstride (ink_pixbuf);
-		ps = ink_bitmap.buffer + by * ink_bitmap.pitch;
-		for (bx = 0; bx < ink_bitmap.width; bx++) {
-			*pd++ = r;
-		        *pd++ = g;
-			*pd++ = b;
-			*pd++ = *ps++;
-		}
-	}
-
-	g_free (ink_bitmap.buffer);
-
-	gdk_pixbuf_composite (ink_pixbuf, pixbuf,
-			      ink_rect.x, ink_rect.y,
-			      MIN (region.x1 - region.x0, ink_bitmap.width),
-			      MIN (region.y1 - region.y0, ink_bitmap.rows),
-			      ink_rect.x, ink_rect.y,
-			      1.0, 1.0, GDK_INTERP_BILINEAR, 0xFF);
-
-	g_object_unref (ink_pixbuf);
-}
-
-/**
- * eel_gdk_pixbuf_draw_layout
- * @pixbuf:    the pixbuf on which to draw the layout
- * @x:         the X position of the left of the layout (in pixels)
- * @y:         the Y position of the top of the layout (in pixels)
- * @color:     the color to use to draw the text, in "eel RGB" format
- * @layout:    a #PangoLayout
- *
- * Render a #PangoLayout onto a #GdkPixbuf.
- */
-void
-eel_gdk_pixbuf_draw_layout (GdkPixbuf *pixbuf,
-			    int x, int y,
-			    guint32 color,
-			    PangoLayout *layout)
-{
-	ArtIRect region;
-
-	region.x0 = x;
-	region.y0 = y;
-	region.x1 = region.y1 = G_MAXINT;
-
-	eel_gdk_pixbuf_draw_layout_clipped (pixbuf, region, color, layout);
-}
-
 GdkPixbuf *
 eel_gdk_pixbuf_scale_down (GdkPixbuf *pixbuf,
 			   int dest_width,
@@ -1568,7 +1230,7 @@ void
 eel_self_check_gdk_pixbuf_extensions (void)
 {
 	GdkPixbuf *pixbuf;
-	ArtIRect clip_area;
+	EelIRect clip_area;
 
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, 100, 100);
 
@@ -1579,40 +1241,40 @@ eel_self_check_gdk_pixbuf_extensions (void)
 
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, eel_gdk_pixbuf_whole_pixbuf), 0, 0, 100, 100);
 
-	clip_area = eel_art_irect_assign (0, 0, 0, 0);
+	clip_area = eel_irect_assign (0, 0, 0, 0);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 0, 0);
 
-	clip_area = eel_art_irect_assign (0, 0, 0, 0);
+	clip_area = eel_irect_assign (0, 0, 0, 0);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 0, 0);
 
-	clip_area = eel_art_irect_assign (0, 0, 100, 100);
+	clip_area = eel_irect_assign (0, 0, 100, 100);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 100, 100);
 
-	clip_area = eel_art_irect_assign (-10, -10, 100, 100);
+	clip_area = eel_irect_assign (-10, -10, 100, 100);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 90, 90);
 
-	clip_area = eel_art_irect_assign (-10, -10, 110, 110);
+	clip_area = eel_irect_assign (-10, -10, 110, 110);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 100, 100);
 
-	clip_area = eel_art_irect_assign (0, 0, 99, 99);
+	clip_area = eel_irect_assign (0, 0, 99, 99);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 99, 99);
 
-	clip_area = eel_art_irect_assign (0, 0, 1, 1);
+	clip_area = eel_irect_assign (0, 0, 1, 1);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 1, 1);
 
-	clip_area = eel_art_irect_assign (-1, -1, 1, 1);
+	clip_area = eel_irect_assign (-1, -1, 1, 1);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 0, 0);
 
-	clip_area = eel_art_irect_assign (-1, -1, 2, 2);
+	clip_area = eel_irect_assign (-1, -1, 2, 2);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 1, 1);
 
-	clip_area = eel_art_irect_assign (100, 100, 1, 1);
+	clip_area = eel_irect_assign (100, 100, 1, 1);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 0, 0);
 
-	clip_area = eel_art_irect_assign (101, 101, 1, 1);
+	clip_area = eel_irect_assign (101, 101, 1, 1);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 0, 0, 0, 0);
 
-	clip_area = eel_art_irect_assign (80, 0, 100, 100);
+	clip_area = eel_irect_assign (80, 0, 100, 100);
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 80, 0, 100, 100);
 
 	g_object_unref (pixbuf);
